#ifndef __B2_CIRCLE_SHAPE_H__
#define __B2_CIRCLE_SHAPE_H__

/// \brief Create b2CircleShape
///
/// @return b2CircleShape instance.
b2CircleShape *b2CircleShape_Create();

/// \brief Destroy b2Circle shape
/// 
/// @param shape The shape instance to destroy. The instance becomes invalid.
void b2CircleShape_Destroy(b2CircleShape *shape);

/// \brief Clone a b2CircleShape.
///
/// Implement b2Shape.
/// @param shape The shape instance to be cloned.
/// @param allocator The b2BlockAllocator to be used.
/// @return b2Shape The clone shape.
b2Shape* b2CircleShape_Clone(b2CircleShape *shape, b2BlockAllocator* allocator);

/// \brief Test if a point is contained inside a shape.
///
/// Implement b2Shape.
/// @param shape The shape instance.
/// @param transform The b2Transform to be used.
/// @param p The point to be tested. 
/// @return 1 if the point is contained in the shape, 0 otherwise.
var b2CircleShape_TestPoint(b2CircleShape *shape, const b2Transform *transform, const VECTOR *p);

/// \brief Perform a raycast test with a b2CircleShape
///
/// Implement b2Shape.
/// @param shape The shape instance.
/// @param output A pre initialized b2RayCastOutput to receive the result.
/// @param input The raycast input parameters.
/// @return 1 if the raycast succeded, 0 otherwise.
var b2CircleShape_RayCast(b2CircleShape *shape, b2RayCastOutput* output, const b2RayCastInput *input, const b2Transform *transform);

/// \brief Compute shape AABB
///
/// @see b2Shape_ComputeAABB
///
/// @param shape The b2Shape instance.
/// @param aabb returns the axis aligned box.
/// @param xf the world transform of the shape.
void b2CircleShape_ComputeAABB(b2CircleShape *shape, b2AABB* aabb, const b2Transform *transform);

/// \brief Compute shape mass
///
/// The inertia tensor is computed about the local origin.
/// @see b2Shape_ComputeMass
///
/// @param shape The b2Shape instance.
/// @param massData returns the mass data for this shape.
/// @param density the density in kilograms per meter squared.
void b2CircleShape_ComputeMass(b2CircleShape *shape, b2MassData* massData, var density);

/// \brief Get the supporting vertex index in the given direction.
///
/// @param shape The b2Shape instance.
/// @param d The direction vector.
/// @return The supporting vertex index.
var b2CircleShape_GetSupport(b2CircleShape *shape, const VECTOR *d);

/// \brief Get the supporting vertex in the given direction.
///
/// @param shape The b2Shape instance.
/// @param d The direction vector.
/// @param outSupport A pre-initialized VECTOR to receive the support vertex.
void b2CircleShape_GetSupportVertex(b2CircleShape *shape, const VECTOR *d, VECTOR *outSupport);

/// \brief Get the vertex count.
///
/// @param shape The shape index.
/// @return The vertex count
var b2CircleShape_GetVertexCount(b2CircleShape *shape);

/// \brief Get a vertex by index. Used by b2Distance.
///
/// @param shape The b2Shape instance.
/// @param index The vertex index.
/// @param outVertex A pre-initialized VECTOR to receive the vertex position.
void b2CircleShape_GetVertex(b2CircleShape *shape, var index, VECTOR *outVertex);

/// \brief Set Position
///
/// @param shape The b2Shape instance.
/// @param position The desired position as a vector.
void b2CircleShape_SetPosition(b2CircleShape *shape, const VECTOR *position);

/// \brief Get Position
///
/// @param shape The shape instance.
/// @param outPosition A pre-initialized vector to receive the position.
void b2CircleShape_GetPosition(b2CircleShape *shape, VECTOR *outPosition);

/// \brief Set type
///
/// Set the b2Shape type. 
/// @param shape The shape.
/// @param type The new type.
#define b2CircleShape_SetType(shape, type) b2Shape_SetType(shape, type)

/// \brief Get shape type.
///
/// @param shape The shape.
/// @return The shape type
///			- E_UNKNOWN
///			- E_CIRCLE
///			- E_POLYGON
///			- E_TYPECOUNT
#define b2CircleShape_GetType(shape) b2Shape_GetType(shape)

/// \brief Set shape radius.
///
/// @param shape The shape.
/// @param radius The new radius.
#define b2CircleShape_SetRadius(shape, radius) b2Shape_SetRadius(shape, radius)

/// \brief Get shape radius.
///
/// @param shape The shape.
/// @return The shape radius.
#define b2CircleShape_GetRadius(shape) b2Shape_GetRadius(shape)

#endif //__B2_CIRCLE_SHAPE_H__
