#ifndef __B2_SHAPE_H__
#define __B2_SHAPE_H__

/// \brief Call shape destructor
///
/// Release the memory occupied by the b2Shape instance b2CircleShape ou b2PolygonShape
/// @param shape b2Shape instance to be released.
void b2Shape_Destroy(b2Shape *shape);

/// \brief Clone the concrete shape using the provided allocator.
///
/// @param shape b2Shape instance to be manipulated.
/// @param allocator b2Allocator instance.
/// @return A new b2Shape instance cloned from the first.
b2Shape* b2Shape_Clone(b2Shape *shape, b2BlockAllocator* allocator);

/// \brief Get the type of this shape. You can use this to down cast to the concrete shape.
///
/// @param shape b2Shape instance.
/// @return the shape type.
var b2Shape_GetType(b2Shape *shape);

/// \brief Test a point for containment in this shape.
///
/// This only works for convex shapes.
/// @param shape b2Shape instance.
/// @param xf the shape world transform.
/// @param p a point in world coordinates.
var b2Shape_TestPoint(b2Shape *shape, const b2Transform *xf, const VECTOR *p);

/// \brief Cast a ray against this shape.
///
/// @param shape The b2Shape instance.
/// @param output The ray-cast results.
/// @param input The ray-cast input parameters.
/// @param transform The transform to be applied to the shape.
var b2Shape_RayCast(b2Shape *shape, b2RayCastOutput* output, const b2RayCastInput *input, const b2Transform *transform);

/// \brief Given a transform, compute the associated axis aligned bounding box for this shape.
/// 
/// @param shape The b2Shape instance.
/// @param aabb returns the axis aligned box.
/// @param xf the world transform of the shape.
void b2Shape_ComputeAABB(b2Shape *shape, b2AABB* aabb, const b2Transform *xf);

/// \brief Compute the mass properties of this shape using its dimensions and density.
///
/// The inertia tensor is computed about the local origin.
/// @param shape The b2Shape instance.
/// @param massData returns the mass data for this shape.
/// @param density the density in kilograms per meter squared.
void b2Shape_ComputeMass(b2Shape *shape, b2MassData* massData, var density);

/// \brief Set type
///
/// Set the b2Shape type. 
/// @param shape The shape.
/// @param type The new type.
void b2Shape_SetType(b2Shape *shape, var type);

/// \brief Get shape type.
///
/// @param shape The shape.
/// @return The shape type
///			- E_UNKNOWN
///			- E_CIRCLE
///			- E_POLYGON
///			- E_TYPECOUNT
var b2Shape_GetType(b2Shape *shape);

/// \brief Set shape radius.
///
/// @param shape The shape.
/// @param radius The new radius.
void b2Shape_SetRadius(b2Shape *shape, var radius);

/// \brief Get shape radius.
///
/// @param shape The shape.
/// @return The shape radius.
var b2Shape_GetRadius(b2Shape *shape);

#endif //__B2_SHAPE_H__
