#ifndef __B2_TYPES_H__
#define __B2_TYPES_H__

//////////////////////////////
// Dynamics
//////////////////////////////

/// A fixture definition is used to create a fixture. This class defines an
/// abstract fixture definition. You can reuse fixture definitions safely.
typedef struct b2FixtureDef
{
	void *dummy;
} b2FixtureDef;

/// A fixture is used to attach a shape to a body for collision detection. A fixture
/// inherits its transform from its parent. Fixtures hold additional non-geometric data
/// such as friction, collision filters, etc.
/// Fixtures are created via b2Body::CreateFixture.
/// @warning you cannot reuse fixtures.
typedef struct b2Fixture
{
	void *dummy;
} b2Fixture;

/// The body type.
/// static: zero mass, zero velocity, may be manually moved
/// kinematic: zero mass, non-zero velocity set by user, moved by solver
/// dynamic: positive mass, non-zero velocity determined by forces, moved by solver
typedef struct b2BodyDef
{
	void *dummy;
} b2BodyDef;

/// A rigid body. These are created via b2World::CreateBody.
typedef struct b2Body
{
	void *dummy;
} b2Body;

/// The world class manages all physics entities, dynamic simulation,
/// and asynchronous queries. The world also contains efficient memory
/// management facilities.
typedef struct b2World
{
	void *dummy;
} b2World;

/// This holds contact filtering data.
/*typedef struct b2Filter
{
	void *dummy;
} b2Filter;*/

/// This holds contact filtering data.
typedef struct b2Filter
{
	/// The collision category bits. Normally you would just set one bit.
	uint16 categoryBits;

	/// The collision mask bits. This states the categories that this
	/// shape would accept for collision.
	uint16 maskBits;

	/// Collision groups allow a certain group of objects to never collide (negative)
	/// or always collide (positive). Zero means no collision group. Non-zero group
	/// filtering always wins against the mask bits.
	uint16 groupIndex;	
} b2Filter;

//////////////////////////////
// Dynamics/Callbacks
//////////////////////////////

/// Joints and fixtures are destroyed when their associated
/// body is destroyed. Implement this listener so that you
/// may nullify references to these joints and shapes.
typedef struct b2DestructionListener
{
	void *jointCallback;
	void *fixtureCallback;
	void *userData;
} b2DestructionListener;

/// Implement this class to provide collision filtering. In other words, you can implement
/// this class if you want finer control over contact creation.
typedef struct b2ContactFilter
{
	void *filterCallback;
	void *userData;
} b2ContactFilter;

/// Implement this class to get contact information. You can use these results for
/// things like sounds and game logic. You can also get contact results by
/// traversing the contact lists after the time step. However, you might miss
/// some contacts because continuous physics leads to sub-stepping.
/// Additionally you may receive multiple callbacks for the same contact in a
/// single time step.
/// You should strive to make your callbacks efficient because there may be
/// many callbacks per time step.
/// @warning You cannot create/destroy Box2D entities inside these callbacks.
typedef struct b2ContactListener
{
	void *beginContactCallback;
	void *endContactCallback;
	void *presolveCallback;
	void *postSolveCallback;
	void *userData;
} b2ContactListener;

/// Callback class for AABB queries.
/// See b2World::Query
typedef struct b2QueryCallback
{
	void *reportFixtureCallback;
	void *userData;
} b2QueryCallback;

/// Callback class for ray casts.
/// See b2World::RayCast
typedef struct b2RayCastCallback
{
	void *reportFixtureCallback;
	void *userData;
} b2RayCastCallback;

/// Implement and register this class with a b2World to provide debug drawing of physics
/// entities in your game.
typedef struct b2DebugDraw
{
	/// Draw a closed polygon provided in CCW order.
	void *drawPolygonFunc;
	/// Draw a solid closed polygon provided in CCW order.
	void *drawSolidPolygonFunc;
	/// Draw a circle.
	void *drawCircleFunc;
	/// Draw a solid circle.
	void *drawSolidCircleFunc;
	/// Draw a line segment.
	void *drawSegmentFunc;
	/// Draw a transform. Choose your own length scale.
	/// @param xf a transform.
	void *drawTransformFunc;

	void *userData;
} b2DebugDraw;

/// \brief Default debug draw function for use with lite - C
///
/// It must be created with the corresponding creator function
typedef struct ackDebugDraw
{
	var world_scale;
	var alpha;
} ackDebugDraw;

//////////////////////////////
// Dynamics/Contacts
//////////////////////////////

typedef struct b2Contact
{
	void *dummy;
} b2Contact;

/// A contact edge is used to connect bodies and contacts together
/// in a contact graph where each body is a node and each contact
/// is an edge. A contact edge belongs to a doubly linked list
/// maintained in each attached body. Each contact has two contact
/// nodes, one for each attached body.
typedef struct b2ContactEdge
{
	b2Body* other;			///< provides quick access to the other body attached.
	b2Contact* contact;		///< the contact
	struct b2ContactEdge* prev;	///< the previous contact edge in the body's contact list
	struct b2ContactEdge* next;	///< the next contact edge in the body's contact list
} b2ContactEdge;

//////////////////////////////
// Dynamics/Joints
//////////////////////////////

/// Joint definitions are used to construct joints.
typedef struct b2JointDef
{
	void *dummy;
} b2JointDef;

/// The base joint class. Joints are used to constraint two bodies together in
/// various fashions. Some joints also feature limits and motors.
typedef struct b2Joint
{
	void *dummy;
} b2Joint;

/// A joint edge is used to connect bodies and joints together
/// in a joint graph where each body is a node and each joint
/// is an edge. A joint edge belongs to a doubly linked list
/// maintained in each attached body. Each joint has two joint
/// nodes, one for each attached body.
typedef struct b2JointEdge
{
	b2Body* other;			///< provides quick access to the other body attached.
	b2Joint* joint;			///< the joint
	struct b2JointEdge* prev;		///< the previous joint edge in the body's joint list
	struct b2JointEdge* next;		///< the next joint edge in the body's joint list
} b2JointEdge;

/// Line joint definition. This requires defining a line of
/// motion using an axis and an anchor point. The definition uses local
/// anchor points and a local axis so that the initial configuration
/// can violate the constraint slightly. The joint translation is zero
/// when the local anchor points coincide in world space. Using local
/// anchors and a local axis helps when saving and loading a game.
typedef struct b2LineJointDef
{
	void *dummy;
} b2LineJointDef;

/// A line joint. This joint provides two degrees of freedom: translation
/// along an axis fixed in body1 and rotation in the plane. You can use a
/// joint limit to restrict the range of motion and a joint motor to drive
/// the motion or to model joint friction.
typedef struct b2LineJoint
{
	void *dummy;
} b2LineJoint;

/// Gear joint definition. This definition requires two existing
/// revolute or prismatic joints (any combination will work).
/// The provided joints must attach a dynamic body to a static body.
typedef struct b2GearJointDef
{
	void *dummy;
} b2GearJointDef;

/// A gear joint is used to connect two joints together. Either joint
/// can be a revolute or prismatic joint. You specify a gear ratio
/// to bind the motions together:
/// coordinate1 + ratio * coordinate2 = constant
/// The ratio can be negative or positive. If one joint is a revolute joint
/// and the other joint is a prismatic joint, then the ratio will have units
/// of length or units of 1/length.
/// @warning The revolute and prismatic joints must be attached to
/// fixed bodies (which must be body1 on those joints).
typedef struct b2GearJoint
{
	void *dummy;
} b2GearJoint;

/// Friction joint definition.
typedef struct b2FrictionJointDef
{
	void *dummy;
} b2FrictionJointDef;

/// Friction joint. This is used for top-down friction.
/// It provides 2D translational friction and angular friction.
typedef struct b2FrictionJoint
{
	void *dummy;
} b2FrictionJoint;

/// Distance joint definition. This requires defining an
/// anchor point on both bodies and the non-zero length of the
/// distance joint. The definition uses local anchor points
/// so that the initial configuration can violate the constraint
/// slightly. This helps when saving and loading a game.
/// @warning Do not use a zero or short length.
typedef struct b2DistanceJointDef
{
	void *dummy;
} b2DistanceJointDef;

/// A distance joint constrains two points on two bodies
/// to remain at a fixed distance from each other. You can view
/// this as a massless, rigid rod.
typedef struct b2DistanceJoint
{
	void *dummy;
} b2DistanceJoint;

/// Mouse joint definition. This requires a world target point,
/// tuning parameters, and the time step.
typedef struct b2MouseJointDef
{
	void *dummy;
} b2MouseJointDef;

/// A mouse joint is used to make a point on a body track a
/// specified world point. This a soft constraint with a maximum
/// force. This allows the constraint to stretch and without
/// applying huge forces.
/// NOTE: this joint is not documented in the manual because it was
/// developed to be used in the testbed. If you want to learn how to
/// use the mouse joint, look at the testbed.
typedef struct b2MouseJoint
{
	void *dummy;
} b2MouseJoint;

/// Prismatic joint definition. This requires defining a line of
/// motion using an axis and an anchor point. The definition uses local
/// anchor points and a local axis so that the initial configuration
/// can violate the constraint slightly. The joint translation is zero
/// when the local anchor points coincide in world space. Using local
/// anchors and a local axis helps when saving and loading a game.
/// @warning at least one body should by dynamic with a non-fixed rotation.
typedef struct b2PrismaticJointDef
{
	void *dummy;
} b2PrismaticJointDef;

/// A prismatic joint. This joint provides one degree of freedom: translation
/// along an axis fixed in body1. Relative rotation is prevented. You can
/// use a joint limit to restrict the range of motion and a joint motor to
/// drive the motion or to model joint friction.
typedef struct b2PrismaticJoint
{
	void *dummy;
} b2PrismaticJoint;

/// Pulley joint definition. This requires two ground anchors,
/// two dynamic body anchor points, max lengths for each side,
/// and a pulley ratio.
typedef struct b2PulleyJointDef
{
	void *dummy;
} b2PulleyJointDef;

/// The pulley joint is connected to two bodies and two fixed ground points.
/// The pulley supports a ratio such that:
/// length1 + ratio * length2 <= constant
/// Yes, the force transmitted is scaled by the ratio.
/// The pulley also enforces a maximum length limit on both sides. This is
/// useful to prevent one side of the pulley hitting the top.
typedef struct b2PulleyJoint
{
	void *dummy;
} b2PulleyJoint;

/// Revolute joint definition. This requires defining an
/// anchor point where the bodies are joined. The definition
/// uses local anchor points so that the initial configuration
/// can violate the constraint slightly. You also need to
/// specify the initial relative angle for joint limits. This
/// helps when saving and loading a game.
/// The local anchor points are measured from the body's origin
/// rather than the center of mass because:
/// 1. you might not know where the center of mass will be.
/// 2. if you add/remove shapes from a body and recompute the mass,
///    the joints will be broken.
typedef struct b2RevoluteJointDef
{
	void *dummy;
} b2RevoluteJointDef;

/// A revolute joint constrains two bodies to share a common point while they
/// are free to rotate about the point. The relative rotation about the shared
/// point is the joint angle. You can limit the relative rotation with
/// a joint limit that specifies a lower and upper angle. You can use a motor
/// to drive the relative rotation about the shared point. A maximum motor torque
/// is provided so that infinite forces are not generated.
typedef struct b2RevoluteJoint
{
	void *dummy;
} b2RevoluteJoint;

/// Weld joint definition. You need to specify local anchor points
/// where they are attached and the relative body angle. The position
/// of the anchor points is important for computing the reaction torque.
typedef struct b2WeldJointDef
{
	void *dummy;
} b2WeldJointDef;

/// A weld joint essentially glues two bodies together. A weld joint may
/// distort somewhat because the island constraint solver is approximate.
typedef struct b2WeldJoint
{
	void *dummy;
} b2WeldJoint;

//////////////////////////////
// Collision
//////////////////////////////

/// Ray-cast input data. The ray extends from p1 to p1 + maxFraction * (p2 - p1).
typedef struct b2RayCastInput
{
	VECTOR p1, p2;
	var maxFraction;
} b2RayCastInput;

/// Ray-cast output data. The ray hits at p1 + fraction * (p2 - p1), where p1 and p2
/// come from b2RayCastInput.
typedef struct b2RayCastOutput
{
	VECTOR normal;
	var fraction;
} b2RayCastOutput;

/// An axis aligned bounding box.
typedef struct b2AABB
{
	void *dummy;
} b2AABB;

/// This holds the mass data computed for a shape.
typedef struct b2MassData
{
	/// The mass of the shape, usually in kilograms.
	var mass;

	/// The position of the shape's centroid relative to the shape's origin.
	VECTOR center;

	/// The rotational inertia of the shape about the local origin.
	var I;
} b2MassData;

/// A distance proxy is used by the GJK algorithm.
/// It encapsulates any shape
typedef struct b2DistanceProxy
{
	void *dummy;
} b2DistanceProxy;

/// \brief Input parameters for b2TimeOfImpact
///
/// Not directly useable in 3DGD because of its methods
/// All members must be accessed through corresponding functions
typedef struct b2TOIInput
{
	void *dummy;
} b2TOIInput;

/// \brief Output parameters for b2TimeOfImpact.
///
/// Can be used directly in 3DGS
typedef struct b2TOIOutput
{
	/// The current state
	/// One of E_UNKNOWN, E_FAILED, E_OVERLAPPED, E_TOUCHING, E_SEPARATED
	var state;
	
	/// The time
	var t;
} b2TOIOutput;

/// \brief Used to warm start b2Distance.
///
/// Set count to zero on first call.
typedef struct b2SimplexCache
{
	float32 metric;		///< length or area
	uint16 count;
	uint8 indexA[3];	///< vertices on shape A
	uint8 indexB[3];	///< vertices on shape B
} b2SimplexCache;

/// \brief Input for b2Distance.
///
/// Not directly useable in 3DGD because of its member complexity
/// All members must be accessed through corresponding functions
/// You have to option to use the shape radii
/// in the computation. Even 
typedef struct b2DistanceInput
{
	void *dummy;
} b2DistanceInput;

/// \brief Acknex version of output for b2Distance.
///
/// This structure can be used directly
typedef struct b2DistanceOutput
{
	VECTOR pointA;		///< closest point on shapeA
	VECTOR pointB;		///< closest point on shapeB
	var distance;
	var iterations;	///< number of GJK iterations used
} b2DistanceOutput;

/// A manifold point is a contact point belonging to a contact
/// manifold. It holds details related to the geometry and dynamics
/// of the contact points.
/// The local point usage depends on the manifold type:
/// -e_circles: the local center of circleB
/// -e_faceA: the local center of cirlceB or the clip point of polygonB
/// -e_faceB: the clip point of polygonA
/// This structure is stored across time steps, so we keep it small.
/// Note: the impulses are used for internal caching and may not
/// provide reliable contact forces, especially for high speed collisions.
typedef struct b2ManifoldPoint
{
	void *dummy;	
} b2ManifoldPoint;

/// A manifold for two touching convex shapes.
/// Box2D supports multiple types of contact:
/// - clip point versus plane with radius
/// - point versus point with radius (circles)
/// The local point usage depends on the manifold type:
/// -e_circles: the local center of circleA
/// -e_faceA: the center of faceA
/// -e_faceB: the center of faceB
/// Similarly the local normal usage:
/// -e_circles: not used
/// -e_faceA: the normal on polygonA
/// -e_faceB: the normal on polygonB
/// We store contacts in this way so that position correction can
/// account for movement, which is critical for continuous physics.
/// All contact scenarios must be expressed in one of these types.
/// This structure is stored across time steps, so we keep it small.
typedef struct b2Manifold
{
	void *dummy;
} b2Manifold;

/// This is used to compute the current state of a contact manifold.
typedef struct b2WorldManifold
{
	void *dummy;
} b2WorldManifold;

/// Used for computing contact manifolds.
typedef struct b2ClipVertex
{
	void *dummy;
} b2ClipVertex;

//////////////////////////////
// Collision/Shapes
//////////////////////////////

/// A shape is used for collision detection. You can create a shape however you like.
/// Shapes used for simulation in b2World are created automatically when a b2Fixture
/// is created.
typedef struct b2Shape
{
	void *dummy;
} b2Shape;

/// A circle shape.
typedef struct b2CircleShape
{
	void *dummy;
} b2CircleShape;

/// A convex polygon. It is assumed that the interior of the polygon is to
/// the left of each edge.
typedef struct b2PolygonShape
{
	void *dummy;
} b2PolygonShape;


//////////////////////////////
// Common
//////////////////////////////

// This is a stack allocator used for fast per step allocations.
// You must nest allocate/free pairs. The code will assert
// if you try to interleave multiple allocate/free pairs.
typedef struct b2StackAllocator
{
	void *dummy;
} b2StackAllocator;

// This is a small object allocator used for allocating small
// objects that persist for more than one time step.
// See: http://www.codeproject.com/useritems/Small_Block_Allocator.asp
typedef struct b2BlockAllocator
{
	void *dummy;
} b2BlockAllocator;

/// A transform contains translation and rotation. It is used to represent
/// the position and orientation of rigid frames.
typedef struct b2Transform
{
	void *dummy;
} b2Transform;

/// This describes the motion of a body/shape for TOI computation.
/// Shapes are defined with respect to the body origin, which may
/// no coincide with the center of mass. However, to support dynamics
/// we must interpolate the center of mass position.
typedef struct b2Sweep
{
	void *dummy;
} b2Sweep;

#endif
