#ifndef __B2_BODY_H__
#define __B2_BODY_H__

/// Creates a fixture and attach it to this body. Use this function if you need
/// to set some fixture parameters, like friction. Otherwise you can create the
/// fixture directly from a shape.
/// If the density is non-zero, this function automatically updates the mass of the body.
/// Contacts are not created until the next time step.
/// @param def the fixture definition.
/// @warning This function is locked during callbacks.
b2Fixture* b2Body_CreateFixture(b2Body *body, const b2FixtureDef* def);

/// Creates a fixture from a shape and attach it to this body.
/// This is a convenience function. Use b2FixtureDef if you need to set parameters
/// like friction, restitution, user data, or filtering.
/// If the density is non-zero, this function automatically updates the mass of the body.
/// @param shape the shape to be cloned.
/// @param density the shape density (set to zero for static bodies).
/// @warning This function is locked during callbacks.
b2Fixture* b2Body_CreateFixture2(b2Body *body, const b2Shape* shape, var density);

/// Destroy a fixture. This removes the fixture from the broad-phase and
/// destroys all contacts associated with this fixture. This will
/// automatically adjust the mass of the body if the body is dynamic and the
/// fixture has positive density.
/// All fixtures attached to a body are implicitly destroyed when the body is destroyed.
/// @param fixture the fixture to be removed.
/// @warning This function is locked during callbacks.
void b2Body_DestroyFixture(b2Body *body, b2Fixture* fixture);

/// Set the position of the body's origin and rotation.
/// This breaks any contacts and wakes the other bodies.
/// Manipulating a body's transform may cause non-physical behavior.
/// @param position the world position of the body's local origin.
/// @param angle the world rotation in radians.
void b2Body_SetTransform(b2Body *body, const VECTOR *position, var angle);

/// Get the body transform for the body's origin.
/// @return the world transform of the body's origin.
void b2Body_GetTransform(b2Body *body, b2Transform *outTransform);

/// Get the world body origin position.
/// @return the world position of the body's origin.
void b2Body_GetPosition(b2Body *body, VECTOR *outPosition);

/// Get the angle in radians.
/// @return the current world rotation angle in radians.
var b2Body_GetAngle(b2Body *body);

/// Get the world position of the center of mass.
void b2Body_GetWorldCenter(b2Body *body, VECTOR *outCenter);

/// Get the local position of the center of mass.
void b2Body_GetLocalCenter(b2Body *body, VECTOR *outCenter);

/// Set the linear velocity of the center of mass.
/// @param v the new linear velocity of the center of mass.
void b2Body_SetLinearVelocity(b2Body *body, const VECTOR *v);

/// Get the linear velocity of the center of mass.
/// @return the linear velocity of the center of mass.
void b2Body_GetLinearVelocity(b2Body *body, VECTOR *velocity);

/// Set the angular velocity.
/// @param omega the new angular velocity in radians/second.
void b2Body_SetAngularVelocity(b2Body *body, var omega);

/// Get the angular velocity.
/// @return the angular velocity in radians/second.
var b2Body_GetAngularVelocity(b2Body *body);

/// Apply a force at a world point. If the force is not
/// applied at the center of mass, it will generate a torque and
/// affect the angular velocity. This wakes up the body.
/// @param force the world force vector, usually in Newtons (N).
/// @param point the world position of the point of application.
void b2Body_ApplyForce(b2Body *body, const VECTOR *force, const VECTOR *point);

/// Apply a torque. This affects the angular velocity
/// without affecting the linear velocity of the center of mass.
/// This wakes up the body.
/// @param torque about the z-axis (out of the screen), usually in N-m.
void b2Body_ApplyTorque(b2Body *body, var torque);

/// Apply an impulse at a point. This immediately modifies the velocity.
/// It also modifies the angular velocity if the point of application
/// is not at the center of mass. This wakes up the body.
/// @param impulse the world impulse vector, usually in N-seconds or kg-m/s.
/// @param point the world position of the point of application.
void b2Body_ApplyLinearImpulse(b2Body *body, const VECTOR *impulse, const VECTOR *point);

/// Apply an angular impulse.
/// @param impulse the angular impulse in units of kg*m*m/s
void b2Body_ApplyAngularImpulse(b2Body *body, var impulse);

/// Get the total mass of the body.
/// @return the mass, usually in kilograms (kg).
var b2Body_GetMass(b2Body *body);

/// Get the rotational inertia of the body about the local origin.
/// @return the rotational inertia, usually in kg-m^2.
var b2Body_GetInertia(b2Body *body);

/// Get the mass data of the body.
/// @return a struct containing the mass, inertia and center of the body.
void b2Body_GetMassData(b2Body *body, b2MassData* data);

/// Set the mass properties to override the mass properties of the fixtures.
/// Note that this changes the center of mass position.
/// Note that creating or destroying fixtures can also alter the mass.
/// This function has no effect if the body isn't dynamic.
/// @param massData the mass properties.
void b2Body_SetMassData(b2Body *body, const b2MassData* data);

/// This resets the mass properties to the sum of the mass properties of the fixtures.
/// This normally does not need to be called unless you called SetMassData to override
/// the mass and you later want to reset the mass.
void b2Body_ResetMassData(b2Body *body);

/// Get the world coordinates of a point given the local coordinates.
/// @param localPoint a point on the body measured relative the the body's origin.
/// @return the same point expressed in world coordinates.
void b2Body_GetWorldPoint(b2Body *body, const VECTOR *localPoint, VECTOR *outWorldPoint);

/// Get the world coordinates of a vector given the local coordinates.
/// @param localVector a vector fixed in the body.
/// @return the same vector expressed in world coordinates.
void b2Body_GetWorldVector(b2Body *body, const VECTOR *localVector, VECTOR *outWorldVector);

/// Gets a local point relative to the body's origin given a world point.
/// @param a point in world coordinates.
/// @return the corresponding local point relative to the body's origin.
void b2Body_GetLocalPoint(b2Body *body, const VECTOR *worldPoint, VECTOR *outLocalPoint);

/// Gets a local vector given a world vector.
/// @param a vector in world coordinates.
/// @return the corresponding local vector.
void b2Body_GetLocalVector(b2Body *body, const VECTOR *worldVector, VECTOR *outLocalVector);

/// Get the world linear velocity of a world point attached to this body.
/// @param a point in world coordinates.
/// @return the world velocity of a point.
void b2Body_GetLinearVelocityFromWorldPoint(b2Body *body, const VECTOR *worldPoint, VECTOR *outLinearVelocity);

/// Get the world velocity of a local point.
/// @param a point in local coordinates.
/// @return the world velocity of a point.
void b2Body_GetLinearVelocityFromLocalPoint(b2Body *body, const VECTOR *localPoint, VECTOR *outLinearVelocity);

/// Get the linear damping of the body.
var b2Body_GetLinearDamping(b2Body *body);

/// Set the linear damping of the body.
void b2Body_SetLinearDamping(b2Body *body, var linearDamping);

/// Get the angular damping of the body.
var b2Body_GetAngularDamping(b2Body *body);

/// Set the angular damping of the body.
void b2Body_SetAngularDamping(b2Body *body, var angularDamping);

/// Set the type of this body. This may alter the mass and velocity.
void b2Body_SetType(b2Body *body, var type);

/// Get the type of this body.
var b2Body_GetType(b2Body *body);

/// Should this body be treated like a bullet for continuous collision detection?
void b2Body_SetBullet(b2Body *body, var flag);

/// Is this body treated like a bullet for continuous collision detection?
var b2Body_IsBullet(b2Body *body);

/// You can disable sleeping on this body. If you disable sleeping, the
/// body will be woken.
void b2Body_SetSleepingAllowed(b2Body *body, var flag);

/// Is this body allowed to sleep
var b2Body_IsSleepingAllowed(b2Body *body);

/// Set the sleep state of the body. A sleeping body has very
/// low CPU cost.
/// @param flag set to true to put body to sleep, false to wake it.
void b2Body_SetAwake(b2Body *body, var flag);

/// Get the sleeping state of this body.
/// @return true if the body is sleeping.
var b2Body_IsAwake(b2Body *body);

/// Set the active state of the body. An inactive body is not
/// simulated and cannot be collided with or woken up.
/// If you pass a flag of true, all fixtures will be added to the
/// broad-phase.
/// If you pass a flag of false, all fixtures will be removed from
/// the broad-phase and all contacts will be destroyed.
/// Fixtures and joints are otherwise unaffected. You may continue
/// to create/destroy fixtures and joints on inactive bodies.
/// Fixtures on an inactive body are implicitly inactive and will
/// not participate in collisions, ray-casts, or queries.
/// Joints connected to an inactive body are implicitly inactive.
/// An inactive body is still owned by a b2World object and remains
/// in the body list.
void b2Body_SetActive(b2Body *body, var flag);

/// Get the active state of the body.
var b2Body_IsActive(b2Body *body);

/// Set this body to have fixed rotation. This causes the mass
/// to be reset.
void b2Body_SetFixedRotation(b2Body *body, var flag);

/// Does this body have fixed rotation?
var b2Body_IsFixedRotation(b2Body *body);

/// Get the list of all fixtures attached to this body.
b2Fixture* b2Body_GetFixtureList(b2Body *body);

/// Get the list of all joints attached to this body.
b2JointEdge* b2Body_GetJointList(b2Body *body);

/// Get the list of all contacts attached to this body.
/// @warning this list changes during the time step and you may
/// miss some collisions if you don't use b2ContactListener.
b2ContactEdge* b2Body_GetContactList(b2Body *body);

/// Get the next body in the world's body list.
b2Body* b2Body_GetNext(b2Body *body);

/// Get the user data pointer that was provided in the body definition.
void* b2Body_GetUserData(b2Body *body);

/// Set the user data. Use this to store your application specific data.
void b2Body_SetUserData(b2Body *body, void* data);

/// Get the parent world of this body.
b2World* b2Body_GetWorld(b2Body *body);

#endif // __B2_BODY_H__
