#ifndef __B2_FIXTURE_H__
#define __B2_FIXTURE_H__

/// Get the type of the child shape. You can use this to down cast to the concrete shape.
/// @return the shape type.
var b2Fixture_GetType(b2Fixture *fixture);

/// Get the child shape. You can modify the child shape, however you should not change the
/// number of vertices because this will crash some collision caching mechanisms.
/// Manipulating the shape may lead to non-physical behavior.
b2Shape* b2Fixture_GetShape(b2Fixture *fixture);

/// Set if this fixture is a sensor.
void b2Fixture_SetSensor(b2Fixture *fixture, var sensor);

/// Is this fixture a sensor (non-solid)?
/// @return the true if the shape is a sensor.
var b2Fixture_IsSensor(b2Fixture *fixture);

/// Set the contact filtering data. This will not update contacts until the next time
/// step when either parent body is active and awake.
void b2Fixture_SetFilterData(b2Fixture *fixture, const b2Filter *filter);

/// Get the contact filtering data.
const b2Filter* b2Fixture_GetFilterData(b2Fixture *fixture);

/// Get the parent body of this fixture. This is NULL if the fixture is not attached.
/// @return the parent body.
b2Body* b2Fixture_GetBody(b2Fixture *fixture);

/// Get the next fixture in the parent body's fixture list.
/// @return the next shape.
b2Fixture* b2Fixture_GetNext(b2Fixture *fixture);

/// Get the user data that was assigned in the fixture definition. Use this to
/// store your application specific data.
void* b2Fixture_GetUserData(b2Fixture *fixture);

/// Set the user data. Use this to store your application specific data.
void b2Fixture_SetUserData(b2Fixture *fixture, void* data);

/// Test a point for containment in this fixture.
/// @param xf the shape world transform.
/// @param p a point in world coordinates.
var b2Fixture_TestPoint(b2Fixture *fixture, const VECTOR *p);

/// Cast a ray against this shape.
/// @param output the ray-cast results.
/// @param input the ray-cast input parameters.
var b2Fixture_RayCast(b2Fixture *fixture, b2RayCastOutput* output, const b2RayCastInput *input);

/// Get the mass data for this fixture. The mass data is based on the density and
/// the shape. The rotational inertia is about the shape's origin. This operation
/// may be expensive.
void b2Fixture_GetMassData(b2Fixture *fixture, b2MassData* massData);

/// Set the density of this fixture. This will _not_ automatically adjust the mass
/// of the body. You must call b2Body::ResetMassData to update the body's mass.
void b2Fixture_SetDensity(b2Fixture *fixture, var density);

/// Get the density of this fixture.
var b2Fixture_GetDensity(b2Fixture *fixture);

/// Get the coefficient of friction.
var b2Fixture_GetFriction(b2Fixture *fixture);

/// Set the coefficient of friction.
void b2Fixture_SetFriction(b2Fixture *fixture, var friction);

/// Get the coefficient of restitution.
var b2Fixture_GetRestitution(b2Fixture *fixture);

/// Set the coefficient of restitution.
void b2Fixture_SetRestitution(b2Fixture *fixture, var restitution);

/// Get the fixture's AABB. This AABB may be enlarge and/or stale.
/// If you need a more accurate AABB, compute it using the shape and
/// the body transform.
const b2AABB* b2Fixture_GetAABB(b2Fixture *fixture);

#endif //__B2_FIXTURE_H__
