#include <acknex.h>
#include <default.c>

#define PRAGMA_PATH "..\\include\\Box2D";
#include "ackb2d.h"

#define PI 3.14159265

// Create a 2d vector
#define vector2(a, b) vector(a, b, 0)

FONT *fnt_instruction = "Arial#14b";
TEXT *txt_instruction = 
{
	string(
			"Press (space) to fire",
			"Press (left/right arrow) to move",
			"Catapult code from : ",
			"http://www.emanueleferonato.com/2010/04/14/real-world-catapult-prototype-using-box2d-%E2%80%93-cleaner-code/#more-2732"
			);
	font = fnt_instruction;
	
	pos_x = 5;
	pos_y = 20;
	flags = SHOW;
}

// Function prototypes
void init();
void cleanup();

void init_debug_draw();
void init_ground();
void init_catapult_body();
void init_catapult_arm();
void init_catapult_motor();
void init_wheels();
void init_wheel_motors();
void init_cannonball();

// Key handler
void handle_keys();
// Set motor speed
void set_motor_speed();

// Draw circle
void polygon (int xc, int yc, double r, int n, COLOR *color);

// Debug Callbacks functions
void draw_polygon(const VECTOR *vertices, var vertexCount, const COLOR *color, void *userData);
void draw_filled_polygon(const VECTOR* vertices, var vertexCount, const COLOR *color, void *userData);
void draw_circle(const VECTOR *center, var radius, const COLOR *color, void *userData);
void draw_filled_circle(const VECTOR *center, var radius, const VECTOR *axis, const COLOR *color, void *userData);
void draw_segment(const VECTOR *p1, const VECTOR *p2, const COLOR *color, void *userData);
void draw_transform(const b2Transform *xf, void *userData);


// Variables
var world_scale = 30;
b2World *world = NULL;
b2Body *cannon_ball = NULL;
b2Body *catapult_chassis_body = NULL;
b2Body *catapult_arm_body = NULL;
b2Body *rear_wheel_body = NULL;
b2Body *front_wheel_body = NULL;
b2RevoluteJoint *arm_revolute_joint = NULL;
b2RevoluteJoint *front_wheel_revolute_joint = NULL;
b2RevoluteJoint *rear_wheel_revolute_joint = NULL;

var pos_x, pos_y = 0;
PANEL *pan_pos = 
{
	digits(0, 0, "%0.2f", "*", 1, pos_x);
	digits(0, 10, "%0.2f", "*", 1, pos_y);
	
	flags = SHOW;
}

///
/// Main function
///
void main()
{
	video_mode = 7;
	wait(1);
	
	// fps max
	fps_max = 60;
	
	// Init world
	init();
	init_debug_draw();
	init_ground();
	init_catapult_body();
	init_catapult_arm();
	init_catapult_motor();
	init_wheels();
	init_wheel_motors();
	init_cannonball();
		
	var velocity_iterations = 10;
	var position_iterations = 10;
	var time_Step = 1.0/60.0;
	
	VECTOR vec_temp;
	
	// Main loop
	while(1)
	{
		b2World_Step(world, time_Step, velocity_iterations, position_iterations);
		b2World_ClearForces(world);
		b2World_DrawDebugData(world);
		b2Body_GetPosition(cannon_ball, &vec_temp);
		
		pos_x = vec_temp.x * world_scale;
		pos_y = vec_temp.y * world_scale;
		
		// Handle keys
		handle_keys();
		
		wait(1);
	}
	
	// Do cleanup tasks
	cleanup();
}

///
/// Do initialization tasks
///
void init()
{
	VECTOR gravity;
	vec_set(&gravity, vector(0, 10.0, 0));
	var do_sleep = 1;
	
	world = b2World_Create(&gravity, do_sleep);
}

///
/// Free all used memories
///
void cleanup()
{
	// Destroy world
	if (world != NULL)
	{
		b2World_Destroy(world);
	}
}

///
/// debug draw
///
void init_debug_draw()
{
	// Create debug draw
	/*b2DebugDraw *debug = b2DebugDraw_Create(draw_polygon, draw_filled_polygon, draw_circle, draw_filled_circle, draw_segment, draw_transform);
	b2DebugDraw_SetFlags(debug, E_SHAPEBIT|E_JOINTBIT);
	
	b2World_SetDebugDraw(world, debug);*/
	
	ackDebugDraw *debug = ackDebugDraw_Create(world_scale);
	b2DebugDraw_SetFlags(debug, E_SHAPEBIT|E_JOINTBIT);
	b2World_SetDebugDraw(world, debug);
}

///
/// ground
///
void init_ground()
{
	b2BodyDef *ground_def = b2BodyDef_Create();
	b2BodyDef_SetPosition(ground_def, vector2(250/world_scale, 400/world_scale));
	
	b2PolygonShape *ground_shape = b2PolygonShape_Create();
	b2PolygonShape_SetAsBox1(ground_shape, 1000/world_scale, 15/world_scale);
	
	b2FixtureDef *ground_fixture = b2FixtureDef_Create();
	b2FixtureDef_SetShape(ground_fixture, ground_shape);
	b2FixtureDef_SetFriction(ground_fixture, 0.9);
	b2FixtureDef_SetRestitution(ground_fixture, 0.1);
	
	b2Body *ground = b2World_CreateBody(world, ground_def);
	b2Body_CreateFixture(ground, ground_fixture);
	
	// Free unused memory
	b2BodyDef_Destroy(ground_def);
	b2Shape_Destroy(ground_shape);
	b2FixtureDef_Destroy(ground_fixture);
}

///
/// catapult body
///
void init_catapult_body()
{
	// Catapult body def
	b2BodyDef *catapult_body_def = b2BodyDef_Create();
	b2BodyDef_SetPosition(catapult_body_def, vector2(350/world_scale, 200/world_scale));
	b2BodyDef_SetType(catapult_body_def, B2_DYNAMICBODY);
	
	// Main part shape
	b2PolygonShape *main_part = b2PolygonShape_Create();
	b2PolygonShape_SetAsBox2(main_part, 125/world_scale, 20/world_scale, vector2(0, 0), 0);
	
	// Main part fixture
	b2FixtureDef *chassis_fixture = b2FixtureDef_Create();
	b2FixtureDef_SetShape(chassis_fixture, main_part);
	b2FixtureDef_SetFriction(chassis_fixture, 0.9);
	b2FixtureDef_SetDensity(chassis_fixture, 50);
	b2FixtureDef_SetRestitution(chassis_fixture, 0.1);
	
	// Fixed Arm shape
	b2PolygonShape *fixed_arm = b2PolygonShape_Create();
	b2PolygonShape_SetAsBox2(fixed_arm, 20/world_scale, 60/world_scale, vector2(-80/world_scale, -40/world_scale), 0);
	
	// Fixed arm fixture
	b2FixtureDef *fixed_arm_fixture = b2FixtureDef_Create();
	b2FixtureDef_SetShape(fixed_arm_fixture, fixed_arm);
	b2FixtureDef_SetFriction(fixed_arm_fixture, 0.9);
	b2FixtureDef_SetDensity(fixed_arm_fixture, 1);
	b2FixtureDef_SetRestitution(fixed_arm_fixture, 0.1);	
	
	// Create bodies
	catapult_chassis_body = b2World_CreateBody(world, catapult_body_def);
	b2Body_CreateFixture(catapult_chassis_body, chassis_fixture);
	b2Body_CreateFixture(catapult_chassis_body, fixed_arm_fixture);
	
	// Destroy unused items
	b2BodyDef_Destroy(catapult_body_def);
	b2PolygonShape_Destroy(main_part);
	b2FixtureDef_Destroy(chassis_fixture);
	
	b2PolygonShape_Destroy(fixed_arm);
	b2FixtureDef_Destroy(fixed_arm_fixture);
}

///
/// Catapult arm
///
void init_catapult_arm()
{
	// Def
	b2BodyDef *catapult_arm_def = b2BodyDef_Create();
	b2BodyDef_SetAllowSleep(catapult_arm_def, 0);
	b2BodyDef_SetPosition(catapult_arm_def, vector2(210/world_scale, 110/world_scale));
	b2BodyDef_SetType(catapult_arm_def, B2_DYNAMICBODY);
	
	// Arm part Shape
	b2PolygonShape *arm_part = b2PolygonShape_Create();
	b2PolygonShape_SetAsBox2(arm_part, 150/world_scale, 10/world_scale, vector2(0, 0), 0);
	
	// Shape fixture
	b2FixtureDef *arm_part_fixture = b2FixtureDef_Create();
	b2FixtureDef_SetShape(arm_part_fixture, arm_part);
	b2FixtureDef_SetFriction(arm_part_fixture, 0.9);
	b2FixtureDef_SetDensity(arm_part_fixture, 5);
	b2FixtureDef_SetRestitution(arm_part_fixture, 0.1);
	
	// Stopper
	b2PolygonShape *stopper = b2PolygonShape_Create();
	b2PolygonShape_SetAsBox2(stopper, 10/world_scale, 20/world_scale, vector2(-140/world_scale, -30/world_scale), 0);
	
	// Stopper fixture
	b2FixtureDef *stopper_fixture = b2FixtureDef_Create();
	b2FixtureDef_SetShape(stopper_fixture, stopper);
	b2FixtureDef_SetFriction(stopper_fixture, 0.9);
	b2FixtureDef_SetDensity(stopper_fixture, 10);
	b2FixtureDef_SetRestitution(stopper_fixture, 0.1);	
	
	// Create body
	catapult_arm_body = b2World_CreateBody(world, catapult_arm_def);
	b2Body_CreateFixture(catapult_arm_body, arm_part_fixture);
	b2Body_CreateFixture(catapult_arm_body, stopper_fixture);
	
	// Destroy unused
	b2BodyDef_Destroy(catapult_arm_def);
	b2PolygonShape_Destroy(arm_part);
	b2FixtureDef_Destroy(arm_part_fixture);
	b2PolygonShape_Destroy(stopper);
	b2FixtureDef_Destroy(stopper_fixture);	
}

///
/// Catapult motors
///
void init_catapult_motor()
{
	b2RevoluteJointDef *arm_joint = b2RevoluteJointDef_Create();
	b2RevoluteJointDef_SetEnableMotor(arm_joint, 1);
	b2RevoluteJointDef_SetEnableLimit(arm_joint, 1);
	b2RevoluteJointDef_Initialize(arm_joint, catapult_chassis_body, catapult_arm_body, vector2(0, 0));
	b2RevoluteJointDef_SetLocalAnchorA(arm_joint, vector2(-80/world_scale, -90/world_scale));
	b2RevoluteJointDef_SetLocalAnchorB(arm_joint, vector2(60/world_scale, 0));
	
	arm_revolute_joint = b2World_CreateJoint(world, arm_joint);
	b2RevoluteJoint_SetMotorSpeed(arm_revolute_joint, 1000);
	b2RevoluteJoint_SetLimits(arm_revolute_joint, -PI, PI/3);
	b2RevoluteJoint_SetMaxMotorTorque(arm_revolute_joint, 1);
	
	// Destroy jointDef
	b2RevoluteJointDef_Destroy(arm_joint);
}

///
/// Init wheels
///
void init_wheels()
{
	b2BodyDef *rear_wheel_def = b2BodyDef_Create();
	b2BodyDef_SetPosition(rear_wheel_def, vector2(250/world_scale, 200/world_scale));
	b2BodyDef_SetType(rear_wheel_def, B2_DYNAMICBODY);
	
	b2CircleShape *rear_wheel_shape = b2CircleShape_Create();
	b2Shape_SetRadius(rear_wheel_shape, 40/world_scale);
	
	b2FixtureDef *rear_wheel_fixture = b2FixtureDef_Create();
	b2FixtureDef_SetShape(rear_wheel_fixture, rear_wheel_shape);
	b2FixtureDef_SetFriction(rear_wheel_fixture, 0.9);
	b2FixtureDef_SetDensity(rear_wheel_fixture, 30);
	b2FixtureDef_SetRestitution(rear_wheel_fixture, 0.1);
	
	// Rear wheel body
	rear_wheel_body = b2World_CreateBody(world, rear_wheel_def);
	b2Body_CreateFixture(rear_wheel_body, rear_wheel_fixture);
	
	// Front wheel
	b2BodyDef *front_wheel_def = b2BodyDef_Create();
	b2BodyDef_SetPosition(front_wheel_def, vector2(450/world_scale, 200/world_scale));
	b2BodyDef_SetType(front_wheel_def, B2_DYNAMICBODY);
	
	b2CircleShape *front_wheel_shape = b2CircleShape_Create();
	b2Shape_SetRadius(front_wheel_shape, 40/world_scale);
	
	b2FixtureDef *front_wheel_fixture = b2FixtureDef_Create();
	b2FixtureDef_SetShape(front_wheel_fixture, front_wheel_shape);
	b2FixtureDef_SetFriction(front_wheel_fixture, 0.9);
	b2FixtureDef_SetDensity(front_wheel_fixture, 30);
	b2FixtureDef_SetRestitution(front_wheel_fixture, 0.1);
	
	// front wheel body
	front_wheel_body = b2World_CreateBody(world, front_wheel_def);
	b2Body_CreateFixture(front_wheel_body, front_wheel_fixture);
	
	// Destroy unused objects
	b2BodyDef_Destroy(rear_wheel_def);
	b2CircleShape_Destroy(rear_wheel_shape);
	b2FixtureDef_Destroy(rear_wheel_fixture);
	
	b2BodyDef_Destroy(front_wheel_def);
	b2CircleShape_Destroy(front_wheel_shape);
	b2FixtureDef_Destroy(front_wheel_fixture);
}

///
/// Wheel motors
///
void init_wheel_motors()
{
	// Front wheel
	b2RevoluteJointDef *front_wheel_joint = b2RevoluteJointDef_Create();
	b2RevoluteJointDef_SetEnableMotor(front_wheel_joint, 1);
	b2RevoluteJointDef_Initialize(front_wheel_joint, catapult_chassis_body, front_wheel_body, vector2(0, 0));
	b2RevoluteJointDef_SetLocalAnchorA(front_wheel_joint, vector2(80/world_scale, 0));
	b2RevoluteJointDef_SetLocalAnchorB(front_wheel_joint, vector2(0, 0));
	
	front_wheel_revolute_joint = b2World_CreateJoint(world, front_wheel_joint);
	b2RevoluteJoint_SetMaxMotorTorque(front_wheel_revolute_joint, 1000000);
	
	// Rear wheel
	b2RevoluteJointDef *rear_wheel_joint = b2RevoluteJointDef_Create();
	b2RevoluteJointDef_SetEnableMotor(rear_wheel_joint, 1);
	b2RevoluteJointDef_Initialize(rear_wheel_joint, catapult_chassis_body, rear_wheel_body, vector2(0, 0));
	b2RevoluteJointDef_SetLocalAnchorA(rear_wheel_joint, vector2(-80/world_scale, 0));
	b2RevoluteJointDef_SetLocalAnchorB(rear_wheel_joint, vector2(0, 0));
	
	rear_wheel_revolute_joint = b2World_CreateJoint(world, rear_wheel_joint);
	b2RevoluteJoint_SetMaxMotorTorque(rear_wheel_revolute_joint, 1000000);
}

///
/// Cannonball
///
void init_cannonball()
{
	// Def
	b2BodyDef *cannon_ball_def = b2BodyDef_Create();
	b2BodyDef_SetPosition(cannon_ball_def, vector2(90/world_scale, 90/world_scale));
	b2BodyDef_SetType(cannon_ball_def, B2_DYNAMICBODY);
	
	// Shape
	b2CircleShape *ball = b2CircleShape_Create();
	b2Shape_SetRadius(ball, 10/world_scale);
	
	// Fixture
	b2FixtureDef *cannon_ball_fixture = b2FixtureDef_Create();
	b2FixtureDef_SetShape(cannon_ball_fixture, ball);
	b2FixtureDef_SetFriction(cannon_ball_fixture, 0.9);
	b2FixtureDef_SetDensity(cannon_ball_fixture, 20);
	b2FixtureDef_SetRestitution(cannon_ball_fixture, 0.5);
	
	// Create body
	cannon_ball = b2World_CreateBody(world, cannon_ball_def);
	b2Body_CreateFixture(cannon_ball, cannon_ball_fixture);
	
	// Destroy unused pointers
	b2BodyDef_Destroy(cannon_ball_def);
	b2Shape_Destroy(ball);
	b2FixtureDef_Destroy(cannon_ball_fixture);
}

///
/// Debug draw polygon
///
void draw_polygon(const VECTOR *vertices, var vertexCount, const COLOR *color, void *userData)
{
	// Move to first points
	draw_line(vector2(vertices[0].x * world_scale, vertices[0].y * world_scale), NULL, 100);
	int i = 0;
	for (i = 0; i<vertexCount; i++)
	{
		draw_line(vector2(vertices[i].x * world_scale, vertices[i].y * world_scale), color, 100);	
	}
	draw_line(vector2(vertices[0].x * world_scale, vertices[0].y * world_scale), color, 100);	
}

///
/// Debug draw filled polygon 
///
void draw_filled_polygon(const VECTOR* vertices, var vertexCount, const COLOR *color, void *userData)
{
	// Move to first points
	draw_line(vector2(vertices[0].x * world_scale, vertices[0].y * world_scale), NULL, 100);
	int i = 0;
	for (i = 1; i<vertexCount; i++)
	{
		draw_line(vector2(vertices[i].x * world_scale, vertices[i].y * world_scale), color, 100);	
	}
	draw_line(vector2(vertices[0].x * world_scale, vertices[0].y * world_scale), color, 100);
}

///
/// Debug draw circle
///
void draw_circle(const VECTOR *center, var radius, const COLOR *color, void *userData)
{
	polygon (center->x * world_scale, center->y * world_scale, radius * world_scale, 36, color);
}

///
/// Debug draw filled circle
///
void draw_filled_circle(const VECTOR *center, var radius, const VECTOR *axis, const COLOR *color, void *userData)
{
	polygon (center->x * world_scale, center->y * world_scale, radius * world_scale, 36, color);
	
	// Draw axis
	draw_line(vector2(center->x * world_scale, center->y * world_scale), NULL, 100);
	draw_line(vector2(center->x * world_scale, center->y * world_scale), color, 100);
	draw_line(vector2((center->x + axis->x * radius) * world_scale, (center->y + axis->y * radius) * world_scale), color, 100);
}

///
/// Debug draw segment
///
void draw_segment(const VECTOR *p1, const VECTOR *p2, const COLOR *color, void *userData)
{
	VECTOR temp;
	
	// Draw point 1
	vec_set(&temp, p1);
	vec_mul(&temp, vector2(world_scale, world_scale));
	draw_line(temp, NULL, 100);
	draw_line(temp, color, 100);
	
	// Draw point 2
	vec_set(&temp, p2);
	vec_mul(&temp, vector2(world_scale, world_scale));
	draw_line(temp, color, 100);
}

///
/// Debug draw transform
///
void draw_transform(const b2Transform *xf, void *userData)
{
	VECTOR pos;
	b2Transform_GetPosition(xf, &pos);
	
	VECTOR R[2];
	b2Transform_GetRotationMat(xf, R);
	
	// color 1
	draw_line(vector2(pos.x * world_scale, pos.y * world_scale), NULL, 100);
	draw_line(vector2(pos.x * world_scale, pos.y * world_scale), vector(0, 0, 0xff), 100);
	draw_line(vector2((pos.x + R[0].x) * world_scale, (pos.y + R[0].y) * world_scale), vector(0, 0, 0xff), 100);
	
	// color 2
	draw_line(vector2(pos.x * world_scale, pos.y * world_scale), NULL, 100);
	draw_line(vector2(pos.x * world_scale, pos.y * world_scale), vector(0, 0, 0xff), 100);
	draw_line(vector2((pos.x + R[1].x) * world_scale, (pos.y + R[1].y) * world_scale), vector(0, 0, 0xff), 100);
}

///
/// Draw polygon
///
void polygon (int xc, int yc, double r, int n, COLOR *color) {
    int lastx, lasty;
    
    //double r = sqrt ((x - xc) * (x - xc) + (y - yc) * (y - yc));
    //double a = atan2 (y - yc, x - xc);
    double a = 0;
    int i;
	 int x = xc + r, y = yc;
    for (i = 1; i <= n; i++) {
        lastx = x; lasty = y;
        a = a + PI * 2 / n;
        x = integer ((double)xc + (double)r * cos(a));
        y = integer ((double)yc + (double)r * sin(a));
        //line (lastx, lasty, x, y);
        draw_line(vector(lastx, lasty, 0), NULL, 100);
        draw_line(vector(x, y, 0), color, 100);
    }
}

///
/// Handle keys
///
void handle_keys()
{
	// Fire
	if (key_space)
	{
		b2RevoluteJoint_SetMaxMotorTorque(arm_revolute_joint, 10000);	
	}
	
	// 
	set_motor_speed();
}

///
/// Calculate motor speed
///
void set_motor_speed()
{
	var current_speed = key_cur - key_cul;
	if (!key_cur && !key_cul)
	{
		current_speed = b2RevoluteJoint_GetMotorSpeed(rear_wheel_revolute_joint) * 0.9;
		if (abs(current_speed) < 0.1)
		{
			current_speed = 0;
		}
	}
	
	// Set motor speed
	b2RevoluteJoint_SetMotorSpeed(rear_wheel_revolute_joint, current_speed);
	b2RevoluteJoint_SetMotorSpeed(front_wheel_revolute_joint, current_speed);
}