#ifndef __TEST_H__
#define __TEST_H__

//////////////////////////////
// Default test routines
//////////////////////////////

// Current test name
STRING *str_test_name = "#512";

/// \brief Struct containing all default variables, uses for all tests
/// 
///
typedef struct DEF_VARIABLES
{
	b2World *m_world;	
	b2Body *m_groundBody;
	b2Body *m_bomb;
	b2MouseJoint *m_mouseJoint;
	var m_velocityIterations;
	var m_positionIterations;
	var m_timeStep;
	var m_physScale;
	// world mouse position
	var mouseXWorldPhys;
	var mouseYWorldPhys;
	var mouseXWorld;
	var mouseYWorld;
} DEF_VARIABLES;

/// \brief Struct to be used as the query aabb callback userData
///
/// The found body will be set into the body pointer member
typedef struct PARAM_AABB_CALLBACK
{
	var include_static;
	DEF_VARIABLES *vars;
	b2Body *found_body;
} PARAM_AABB_CALLBACK;

/// \brief last mouse state
var deftest_last_mouse_left = 0;
var deftest_last_d = 0;

/// \brief Defaut test initialization
///
/// Initialize all default test vars variables
/// The struct must be preallocated first
/// @param vars The default test struct to be initialized
void deftest_init(DEF_VARIABLES *vars)
{
	// Init default variables
	vars->m_velocityIterations = 10;
	vars->m_positionIterations = 10;
	vars->m_timeStep = 1.0 / 60.0;
	vars->m_physScale = 30;
	
	vars->m_world = NULL;
	vars->m_groundBody = NULL;
	vars->m_bomb = NULL;
	vars->m_mouseJoint = NULL;
	
	// World vector
	VECTOR gravity; 
	vec_set(&gravity, vec2(0, 10));
	
	// Allow bodies to sleep
	var do_sleep = 1;
	
	// Create world object
	vars->m_world = b2World_Create(&gravity, do_sleep);
	b2World_SetWarmStarting(vars->m_world, 1);
	// Set debug draw to default
	ackDebugDraw *debug = ackDebugDraw_Create(vars->m_physScale);
	ackDebugDraw_SetFlags(debug, E_SHAPEBIT | E_JOINTBIT);
	b2World_SetDebugDraw(vars->m_world, debug);
	
	// Create ground body
	b2BodyDef *ground_body_def = b2BodyDef_Create();
	vars->m_groundBody = b2World_CreateBody(vars->m_world, ground_body_def);
	
	// Create world borders
	b2PolygonShape *wall_shape = 	b2PolygonShape_Create();
	b2BodyDef *wall_def = b2BodyDef_Create();
	b2Body *wall_body = NULL;
	
	// Left
	b2BodyDef_SetPosition(wall_def, vec2(-95 / vars->m_physScale, 360 / vars->m_physScale / 2));
	b2PolygonShape_SetAsBox1(wall_shape, 100 / vars->m_physScale, 400 / vars->m_physScale / 2);
	wall_body = b2World_CreateBody(vars->m_world, wall_def);
	b2Body_CreateFixture2(wall_body, wall_shape, 0);
	
	// Right
	b2BodyDef_SetPosition(wall_def, vec2((640 + 95) / vars->m_physScale, 360 / vars->m_physScale / 2));
	wall_body = b2World_CreateBody(vars->m_world, wall_def);
	b2Body_CreateFixture2(wall_body, wall_shape, 0);
	
	// Top
	b2BodyDef_SetPosition(wall_def, vec2(640 / vars->m_physScale / 2, -95 / vars->m_physScale));
	b2PolygonShape_SetAsBox1(wall_shape, 680 / vars->m_physScale / 2, 100 / vars->m_physScale);
	wall_body = b2World_CreateBody(vars->m_world, wall_def);
	b2Body_CreateFixture2(wall_body, wall_shape, 0);
	
	// Bottom
	b2BodyDef_SetPosition(wall_def, vec2(640 / vars->m_physScale / 2, (screen_size.y + 95) / vars->m_physScale));
	wall_body = b2World_CreateBody(vars->m_world, wall_def);
	b2Body_CreateFixture2(wall_body, wall_shape, 0);
	
	// Delete unused pointers
	b2BodyDef_Destroy(ground_body_def);
	b2PolygonShape_Destroy(wall_shape);
	b2BodyDef_Destroy(wall_def);
	
	// Randomize
	random_seed(0);
}

/// \brief Update mouse position in the box2d simulation
///
/// @param vars The defaut test struct
void deftest_update_mouseworld(DEF_VARIABLES *vars)
{
	vars->mouseXWorldPhys = mouse_cursor.x / vars->m_physScale;
	vars->mouseYWorldPhys = mouse_cursor.y / vars->m_physScale;
	
	vars->mouseXWorld = mouse_cursor.x;
	vars->mouseYWorld = mouse_cursor.y;
}

/// \brief Callback to be used by the queryAABB instruction
///
/// The callback must return 0 to terminate the query and non zero to continue
/// @param fixture The found fixture
var deftest_getbody_callback(b2Fixture *fixture, void *userData)
{
	PARAM_AABB_CALLBACK *param = (PARAM_AABB_CALLBACK *)userData;
	param->found_body = NULL;
	
	b2Shape *shape = b2Fixture_GetShape(fixture);

	b2Body *body = b2Fixture_GetBody(fixture);
	if ((b2Body_GetType(body) != B2_STATICBODY) || (param->include_static == 1))	
	{
		// Initialize transform to be used by b2Body gettransform 
		b2Transform *body_transform = b2Transform_Create();
		b2Body_GetTransform(body, body_transform);
		
		var inside = b2Shape_TestPoint(shape, body_transform, vec2(param->vars->mouseXWorldPhys, param->vars->mouseYWorldPhys));
		
		// Free memoty used by transform
		b2Transform_Destroy(body_transform);
		
		if (inside == 1)
		{
			param->found_body = body;
			// Stop query
			return 0;	
		}
		
		return 1;
	}
}

/// \brief return the body under the mouse
///
/// @param include_static If set to 1, the function also return static bodies
b2Body *deftest_getbody_at_mouse(DEF_VARIABLES *vars, var include_static)
{
	b2AABB *aabb = b2AABB_Create();
	b2AABB_SetLowerBound(aabb, vec2(vars->mouseXWorldPhys - 0.001, vars->mouseYWorldPhys - 0.001));
	b2AABB_SetUpperBound(aabb, vec2(vars->mouseXWorldPhys + 0.001, vars->mouseYWorldPhys + 0.001));
	
	// User data to be used with the queryaabb callback
	PARAM_AABB_CALLBACK params;
	params.include_static = include_static;
	params.vars = vars;
	params.found_body = NULL;
	
	// Create callback
	b2QueryCallback *getbody_callback = b2QueryCallback_Create(deftest_getbody_callback);
	b2QueryCallback_SetUserData(getbody_callback, &params);
	
	// Query world
	b2World_QueryAABB(vars->m_world, getbody_callback, aabb);
	
	// Destroy unused aabb
	b2AABB_Destroy(aabb);
	
	return params.found_body;
}

/// \brief Destroy body at mouse
///
/// @params vars The default test variables
void deftest_mousedestroy(DEF_VARIABLES *vars)
{
	// Mouse down && d pressed
	if ((mouse_left == 0) && (key_d == 1))
	{
		deftest_last_d = key_d; 
	}
	
	// Wait for key release
	if ((deftest_last_d == 1) && (key_d == 0))
	{
		// Disable last key state
		deftest_last_d = key_d;
		
		b2Body *body = deftest_getbody_at_mouse(vars, 1);
		if (body != NULL)
		{
			b2World_DestroyBody(vars->m_world, body);
		}
	}
}

/// \brief Create joint when dragging a body
///
/// @param vars The default test struct
void deftest_mousedrag(DEF_VARIABLES *vars)
{
	if ((mouse_left == 1) && (vars->m_mouseJoint == NULL))
	{
		// Mouse pressed
		deftest_last_mouse_left = mouse_left;
		
		b2Body *body = deftest_getbody_at_mouse(vars, 0);
		if (body != NULL)
		{
			b2MouseJointDef *joint_def =  b2MouseJointDef_Create();
			b2JointDef_SetBodyA(joint_def, vars->m_groundBody);
			b2JointDef_SetBodyB(joint_def, body);
			b2MouseJointDef_SetTarget(joint_def, vec2(vars->mouseXWorldPhys, vars->mouseYWorldPhys));
			b2JointDef_SetCollideConnected(joint_def, 1);
			b2MouseJointDef_SetMaxForce(joint_def, 300 * b2Body_GetMass(body));
			
			// Create joint
			vars->m_mouseJoint = b2World_CreateJoint(vars->m_world, joint_def);
			
			// Awake body
			b2Body_SetAwake(body, 1);
			
			// Destroy joint def
			b2MouseJointDef_Destroy(joint_def);
		}
	}
	
	// Mouse release
	if ((mouse_left == 0) && (deftest_last_mouse_left == 1))
	{
		// reset mouse state
		deftest_last_mouse_left = mouse_left;
		
		if (vars->m_mouseJoint != NULL)
		{
			b2World_DestroyJoint(vars->m_world, vars->m_mouseJoint);
			vars->m_mouseJoint = NULL;	
		}
	}
	
	// Mouse update
	if (vars->m_mouseJoint != NULL)
	{
		b2MouseJoint_SetTarget(vars->m_mouseJoint, vec2(vars->mouseXWorldPhys, vars->mouseYWorldPhys));
	}
}

/// \brief Default test update
///
/// Update Mouse and Draw debug
/// @param vars The default test struct
void deftest_update(DEF_VARIABLES *vars)
{
	// Update mouse joint
	deftest_update_mouseworld(vars);
	deftest_mousedestroy(vars);
	deftest_mousedrag(vars);
	
	// Update physics
	b2World_Step(vars->m_world, vars->m_timeStep, vars->m_velocityIterations, vars->m_positionIterations);
	b2World_ClearForces(vars->m_world);
	
	// Render debug Data
	b2World_DrawDebugData(vars->m_world);
}

/// \brief Default test exit
///
/// Destroy world in current variable
/// @param vars The default test struct
void deftest_exit(DEF_VARIABLES *vars)
{
	// Destroy world
	if (vars->m_world != NULL)
	{
		b2World_Destroy(vars->m_world);
		vars->m_world = NULL;
	}	
}	

/// \brief multiply a matrix with a vector
///
/// @param outVector the result vector
/// @param R array of two vectors (matrix)
/// @param v vector to be multiplied with
void mulmv(VECTOR *outVector,const VECTOR *R, const VECTOR *v)
{
	VECTOR tmp;
	tmp.x = R[0].x * v->x + R[1].x * v->y;
	tmp.y = R[0].y * v->x + R[1].y * v->y;
	tmp.z = 0;
	
	vec_set(outVector, &tmp);
}

/// \brief multiply a transform with a vector
///
/// @param outVector The result vector
/// @param t The transform
/// @param v The vector to be multiplied
void mulx(VECTOR *outVector, const b2Transform *t, const VECTOR *v)
{
	VECTOR tmp;
	VECTOR R[2];
	
	b2Transform_GetRotationMat(t, R);
	mulmv(&tmp, R, v);
	
	VECTOR pos;
	b2Transform_GetPosition(t, &pos);
	
	vec_add(&tmp, &pos);	
	
	vec_set(outVector, &tmp);
}

#endif // __TEST_H__
